
import { promises as fs } from 'fs';
import { join as path_join } from 'path';
import { parse as parse_yaml } from 'yaml';
import { deep_merge } from './utilities/deep-merge';
import { OIDCConfig, validate_oidc_conf } from './security/openid-connect';
import { PKCECookieConfig, validate_pkce_cookie_conf } from './security/pkce-cookie';

const conf_dir = process.env.CONF_PATH;

if (! conf_dir) {
	console.error('No CONF_PATH defined');
	process.exit(1);
}

export async function load_conf() : Promise<unknown> {
	const conf = Object.create(null);
	const files = await fs.readdir(conf_dir, { recursive: true });

	// Sort configuration files by file name for consistent load order (users should
	// number prefix their config file names)
	files.sort((a, b) => a.localeCompare(b));

	// Load each config file in order, parse as YAML, and merge the contents into the
	// results object
	for (const file of files) {
		const conf_yaml = await fs.readFile(path_join(conf_dir, file), 'utf8');
		const conf_parsed = parse_yaml(conf_yaml);

		deep_merge(conf, conf_parsed);
	}

	return conf;
}

export function validate_conf(conf: unknown) : asserts conf is Conf {
	if (typeof conf !== 'object' || ! conf) {
		throw new Error('`conf` is not an object');
	}

	if ('oidc' in conf) {
		validate_oidc_conf(conf.oidc)
	}

	else {
		throw new Error('`conf.oidc` is missing');
	}

	if ('pkce_cookie' in conf) {
		validate_pkce_cookie_conf(conf.pkce_cookie);
	}

	else {
		throw new Error('`conf.pkce_cookie` is missing');
	}

	// todo: validate other config
}

export interface Conf {
	web: {
		address: string;
		port: number;
		exposed_url: string;
		tls?: false | {
			key: string;
			cert: string;
		};
		etag?: {
			static_assets?: 'none' | 'weak' | 'strong';
		};
		cache_control?: {
			static_assets?: string;
		};
	};
	metadata: {
		address: string;
		port: number;
		tls?: false | {
			key: string;
			cert: string;
		};
	};
	oidc: OIDCConfig;
	pkce_cookie: PKCECookieConfig;
	// session_cookie: SessionCookieConfig;
}
